/******************************************************************************
 * $LastChangedDate: 2008-12-15 19:11:02 -0500 (Mon, 15 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Misc functions.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 ******************************************************************************/

#define BASE_MISC_CC 1
#include "base/module.hh"
#include "base/misc.hh"
#include <ctype.h>

namespace base {

/*****************************************************************************
 * Safely process a cmd-line arg.
 * @return Zero if arg idx is invalid.
 *****************************************************************************/
int
Argi( int idx, int argc_, char** argv_ )  // careful, "argv" is a global var in C
{
	if ( idx >= argc_ ) return 0;
	return int(strtol( argv_[idx], NULL, 0 )); // base=0 to convert decimal/hex/octal
}

float
Argf( int idx, int argc_, char** argv_ )
{
	if ( idx >= argc_ ) return 0;
	return atof( argv_[idx] );
}

string
Argstr( int idx, int argc_, char** argv_ )
{
	if ( idx >= argc_ ) return string( "" );
	return string( argv_[idx] );
}

/*****************************************************************************
 * Functions for marking a command-line arg as "done".
 * For helping to catch unrecognized args.
 * @param	i
 *			argv[i]
 * @param	n
 *			How many consecutive args are "done" (default n=1).
 *****************************************************************************/
INTERN bool sCmdargDone[100];

void
CmdargDone( int i, int n )
{
	for ( int j = 0; j < n; ++j ) sCmdargDone[i+j] = true;
}

bool
IfCmdargDone( int i )
{
	return sCmdargDone[i];
}

/*****************************************************************************
 * cygwin lacks strcasestr().
 * This code came from FreeBSD.
 * http://www.freebsd.org/cgi/cvsweb.cgi/src/lib/libc/string/strcasestr.c?rev=1.4;content-type=text%2Fplain
 *****************************************************************************/
#if OS_CYGWIN
char*
STRCASESTR( const char *s, const char* find )
{
    char c, sc;
    size_t len;

    if ((c = *find++) != 0)
    {
        c = tolower((unsigned char)c);
        len = strlen(find);
        do
        {
            do
            {
                if ((sc = *s++) == 0)
                    return (NULL);
            } while ((char)tolower((unsigned char)sc) != c);
        } while (strncasecmp(s, find, len) != 0);
        s--;
    }
    return ((char *)s);
}
#endif // OS_CYGWIN

/*****************************************************************************
 * Print exception.
 *****************************************************************************/
void
PrintException( const Exception& exception, const string& functionName )
{
    CERROR << "** EXCEPTION!! [" << functionName << "] **" << endl
           << exception.GetText() << endl;
}

/*****************************************************************************
 * Convert bytes for printing as "n.nMB".
 *****************************************************************************/
fp
PrintableMegabyte( const long bytes )
{
	// One fractional digit.
	return fp(long(bytes*10) / long(MEGABYTE)) / fp(10.0f);
}

/*****************************************************************************
 * Conver string in-place to lower-case.
 *****************************************************************************/
void
LowerCase( string& s )
{
    for ( uint i = 0; i < s.size(); ++i )
    {
        s[i] = char(tolower(s[i]));
    }
}

/*****************************************************************************
 * Convert string in-place to upper-case.
 *****************************************************************************/
void
UpperCase( string& s )
{
    for ( uint i = 0; i < s.size(); ++i )
    {
        s[i] = char(toupper(s[i]));
    }
}

} // namespace base
