/******************************************************************************
 * $LastChangedDate: 2008-06-05 00:08:30 -0400 (Thu, 05 Jun 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Random number functions.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 ******************************************************************************/

#ifndef BASE_RANDOM_HH
#define BASE_RANDOM_HH 1

#include "base/thread.hh"

namespace base {

// Abbrev.
#define gRandom base::global::randomGenerator  // random.cc (name cannot be "random")

const fp RAND_MAX_RECIP = 1.0 / fp(RAND_MAX);  // a constant (without division instruction)

////////////////////////////////////////////////////////////////////////////////
/// @brief Generates random numbers from a per-thread safe.
///
/// Every thread can set its own seed which is stored in a thread variable.
///
class Random : public Threadable
{
public:
			Random( void );
			~Random();
	void	Init( void );
	void	randomize( uint seed );
	void	randomize( void );
	uint	random_ui( void );
	uint	random_ui( uint range );
	fp		random_f( void );
	fp		random_f( fp range );
	uint	random_ui_true( void );
	uint	GetSeed( void ) { return mSavedSeed; }

	// Fast methods which use a precomputed table but will repeat.
	uint fast_random_ui( void )
	{
		// Table has padding in case multiple threads trample at the end.
		uint r = mFastTable[mFastTableIdx];
		mFastTableIdx = (mFastTableIdx + 1) & FAST_TABLE_IDX_MASK;  // faster than %
		return r;
	}

	uint fast_random_ui( uint range )
	{
		return fast_random_ui() % range;
	}

	fp fast_random_f( void )
	{
		return fp(fast_random_ui()) * RAND_MAX_RECIP;
	}

	fp fast_random_f( fp range )
	{
		return fp(fast_random_ui()) * RAND_MAX_RECIP * range;
	}

private:
	CLASS_CONST uint	FAST_TABLE_CNT      = 1024;  // small table is faster (4K : 1 page)
	CLASS_CONST uint	FAST_TABLE_IDX_MASK = 1023;  // table size must be power-of-2 for mask to be right
	PerThreadPtr<uint0>	mSeed;  // rand_r() will change
	uint				mSavedSeed;
	uint				mFastTable[FAST_TABLE_CNT + 100];
	uint				mFastTableIdx;
};

// Global.
namespace global { extern Random randomGenerator; }

} // namespace base

#endif // BASE_RANDOM_HH
