/*****************************************************************************
 * $LastChangedDate: 2008-12-12 21:44:59 -0500 (Fri, 12 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Collision-detection class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef COLLISION_COLLISION_HH
#define COLLISION_COLLISION_HH 1

#define GET_COLLISION() (Collision::GetInstance())

#include "object/module.hh"
#include "object/events.hh"
using namespace object;

namespace collision {

////////////////////////////////////////////////////////////////////////////////
/// @brief Collision-detection class (Singleton).
///
/// Broadcasts the event of a pair of Objects colliding.
///
/// For factoring, this class is kept simple.
/// Extended actions (the testing of collisions, how to handle them,
/// storing colliders, etc) is delegated to clients (the Dyna class).
///
class Collision
{
PREVENT_COPYING(Collision)
private:
                Collision( void );
                ~Collision();

public:
    CLASS_METHOD Collision& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new Collision;
		return *msInstance;
	}

	void						SetCollidable( shptr<Object> object, const bool collidable );
	pair<bool,shptr0<Object> >	IfCollision( shptr<Dyna> dyna );

private:
	CLASS_DESTROY_OBJECT_LISTENER( Collision, mCollision )

private:
	typedef set<shptr<Object> > Collidables;
    CLASS_VAR Collision*	msInstance;
	Collidables				mCollidables;
};

} // namespace collision

#endif // COLLISION_COLLISION_HH
