/*****************************************************************************
 * $LastChangedDate: 2008-12-02 12:16:59 -0500 (Tue, 02 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Trail particle-system (private implementation of FX class).
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_PARTSYS_TRAIL_HH
#define FX_PARTSYS_TRAIL_HH 1

#include "object/events.hh"
using namespace object;
#include "fx/partsys.hh"
#include "fx/partsys_small.hh"

namespace fx {

////////////////////////////////////////////////////////////////////////////////
/// @brief Trail particle-system (private implementation of FX class).
///
class TrailParticleSystem : public SmallParticleSystem
{
PREVENT_COPYING(TrailParticleSystem)
typedef SmallParticleSystem Parent;
friend class FX; friend class shptr<TrailParticleSystem>;
private:
					TrailParticleSystem( const Milliseconds lifetime,
                                         shptr<Object> trailMaker );
					~TrailParticleSystem();
	virtual void	Tick( const Milliseconds millisecElapsed );
	void			MakeParticles( const uint cnt, const Milliseconds millisecElapsed );

private:
	CLASS_DESTROY_OBJECT_LISTENER( TrailParticleSystem, mTrailParticleSystem )

private:
	shptr<Object>                   mTrailMaker;                ///< emitter of particles
    WorldVertex                     mTrailMakerPrevPos;         ///< prev position of trail maker
	const Milliseconds      		mTimeOfDeath;               ///< time when particle system expires
	Milliseconds        			mTimeOfPrevTick;        	///< previous tick
    shptr<DestroyObjectListener>    mDestroyObjectListener;     ///< listen for the destroying of trail-maker Object
};

} // namespace fx

#endif // FX_PARTSYS_TRAIL_HH
