/*****************************************************************************
 * $LastChangedDate: 2008-08-31 02:29:07 -0400 (Sun, 31 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Texture class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_TEXTURE_HH
#define FX_TEXTURE_HH 1

#include <osg/Texture2D>

namespace fx {

////////////////////////////////////////////////////////////////////////////////
/// @brief Texture class abstracts a texture object, and loads/caches texture image files.
///
/// Texture ctor uses ImageCache.
///
/// For speed:
/// - Clients should call StateSorter::AllocateStateBitsId().
/// - Create an osg::StateSet and bind this Texture to it.
/// - Pass StateSet to World::Attach(node,stateBits,stateSet)
///   in order to group nodes that share this Texture.
///
class Texture : public Shared
{
PREVENT_COPYING(Texture)
public:
				Texture( const string& filename );
        		~Texture();
	void		Bind( osg::StateSet& stateset );  // difficult to abstract osg::StateSet
	CLASS_METHOD RefPtr<osg::StateSet> NewTextureAsStateSet( const string& filename );

private:
	RefPtr<osg::Texture2D>	mTextureObject;
public:
	DECL_TYPESIG(TYPESIG_TEXTURE)
};

} // namespace fx

#endif // FX_TEXTURE_HH
