/*****************************************************************************
 * $LastChangedDate: 2008-12-12 21:44:59 -0500 (Fri, 12 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	A part of the C++ core that calls Lua functions.
 * @remarks Another part is the program module (startup).
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define GLUE_CORE2LUA_CC 1
#include "base/module.hh"
using namespace base;
#include "lua_bind/module.hh"
#include "lua_bind/defs.hh"
using namespace lua;
#include "glue/module.hh"
#include "glue/core2lua.hh"

namespace glue {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  Core2Lua  ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

Core2Lua* Core2Lua::msInstance;

/*****************************************************************************
 * ctor/dtor.
 * Install listeners that propagate events from C++ to Lua.
 *****************************************************************************/
Core2Lua::Core2Lua( void )
{
    EVENT_DESTROY_OBJECT.Listen( new DestroyObjectListener( *this ) );
    EVENT_COLLISION.Listen(      new CollisionListener( *this ) );
}

Core2Lua::~Core2Lua()
{
    // For a Singleton, skip unlistening.
}

/*****************************************************************************
 * Listen for the DestroyObject event and propagate to Lua.
 *****************************************************************************/
void
Core2Lua::DestroyObjectListener::operator()( shptr<Object> object )
{
CHECK_TYPESIG(object,TYPESIG_OBJECT);

    const Lua::Number rawPtr = (Lua::Number) reinterpret_cast<long>( object.PTR() );
    vector<Lua::Number> args;
    args.push_back( rawPtr );
    GET_LUA().CallLuaFunc( LUA_SIM_TABLE_NAME, "HandleDestroyObject", args );
}

/*****************************************************************************
 * Listen for the collision event and propagate to Lua.
 *****************************************************************************/
void
Core2Lua::CollisionListener::operator()( collision::Events::ObjectPair objectPair )
{
CHECK_TYPESIG(objectPair.first,TYPESIG_OBJECT);
CHECK_TYPESIG(objectPair.second,TYPESIG_OBJECT);

    const Lua::Number rawPtr1 = (Lua::Number) reinterpret_cast<long>( objectPair.first.PTR() );
    const Lua::Number rawPtr2 = (Lua::Number) reinterpret_cast<long>( objectPair.second.PTR() );
    vector<Lua::Number> args;
    args.push_back( rawPtr1 );
    args.push_back( rawPtr2 );
    GET_LUA().CallLuaFunc( LUA_SIM_TABLE_NAME, "HandleCollision", args );
}

} // namespace glue
