/******************************************************************************
 * $LastChangedDate: 2008-12-12 21:44:59 -0500 (Fri, 12 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Glue the modules together.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 ******************************************************************************/

#define GLUE_MODULE_CC 1
#include "base/module.hh"
#include "base/misc.hh"
using namespace base;
#include "glue/module.hh"
#include "glue/conf.hh"
#include "glue/sim_time.hh"
#include "glue/core2lua.hh"

namespace glue {

/*****************************************************************************
 * Initialize module.
 *****************************************************************************/
void
Init( const InitArgs& initArgs )
{

BEGIN_ONCE {

    // Initialize Conf struct.
	// Not CONF macro since mValid isn't true yet.
    const string confTxtFilename = "";  // N/A
	glue::global::gConf.Init( initArgs.mArgc,
                              initArgs.mArgv,
                              confTxtFilename,
                              initArgs.mProgramName );  // glue-specific

	for ( int i = 1; i < initArgs.mArgc; ++i )
	{
		if ( STREQ( initArgs.mArgv[i], "-hour" ) )
		{
			// -hour fp (not int)
			++i;  // to arg's value
			const fp hour = Argf( i, initArgs.mArgc, initArgs.mArgv );
			GET_SIM_TIME().SetHour( hour );
			CLOG << "Set hour to " << hour << endl;
		}
	}

    // Glue between C++ core and Lua.
    Core2Lua::GetInstance();  // subtle way to initialize

} END_ONCE

}

/*****************************************************************************
 * Start module.
 *****************************************************************************/
void
Start( void )
{
	// NOP
}

} // namespace glue
