/*****************************************************************************
 * $LastChangedDate: 2008-12-05 10:58:31 -0500 (Fri, 05 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Light class.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef GRAPH_LIGHT_HH
#define GRAPH_LIGHT_HH 1

#include "gfx/module.hh"
#include "gfx/osg.hh"
using namespace gfx;
#include "types/module.hh"
#include "types/rgba.hh"
using namespace types;
namespace view { class View; }
using namespace view;
#include <osg/Light>
#include <osg/LightSource>

namespace graph {

class LightPreDrawViewListener;  // to transform light position

////////////////////////////////////////////////////////////////////////////////
/// @brief Higher-level Light class.
///
/// This Light class allows manipulation as though it were a node.
/// osg::Light isn't a node (it is specially associated with a LightSource).
/// osg::LightSource is a group node.
///
/// See light.cc for issues/difficulties with lighting in OSG.
///
class Light : public Shared
{
PREVENT_COPYING(Light)
friend class LightPreDrawViewListener;
public:
						Light( const uint lightNum,
                               const WorldVertex& lightPos,
                               const RGBA lightColorAmbient,
                               const RGBA lightColorDiffuse,
                               const RGBA lightColorSpecular );
						~Light();
	void				SetPosition( const WorldVertex& pos );
	WorldVertex			GetPosition( void );
	void				SetBaseAmbient( const RGBA color );  // rare
	CLASS_METHOD void	SetDefaultBaseAmbient( const RGBA color );  // rare
	void				SetAmbient( const RGBA color );
	void				SetDiffuse( const RGBA color );
	void				SetSpecular( const RGBA color );
	void				AttachNode( RefPtr<Node> node );
	CLASS_METHOD void	SetMulColor( const fp mulColor );

// LIMITED USE:
	RefPtr<GroupNode>	GetGroupNode( void ) { return mLightSrcGroup.get(); }

private:
	void					PreDraw( shptr<View> view );
	CLASS_METHOD osg::Vec4	MulColor( const RGBA color );

private:
	CLASS_VAR osg::Vec4				msDefaultBaseAmbient;
	CLASS_VAR fp					msMulColor;				///< supports day/night colors
	uint							mLightNum;				///< OpenGL light number
	RefPtr<osg::Light>				mLight;					///< osg::Light isn't a node
	RefPtr<osg::LightSource>		mLightSrcGroup;			///< derivative of osg::Group
	WorldVertex						mWorldPos;				///< position of light in world space
	shptr<LightPreDrawViewListener> mPreDrawViewListener;	///< reflected to event listeners in app
public:
	DECL_TYPESIG(TYPESIG_LIGHT)
};

} // namespace graph

#endif // GRAPH_LIGHT_HH
