/*****************************************************************************
 * $LastChangedDate: 2008-12-05 10:58:31 -0500 (Fri, 05 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	This module contains bootstrap/kickstart routines.
 * @remarks	Some modules define Init()/Start() functions.  Others omit them.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define INIT_MODULE_CC 1
#include "base/module.hh"
using namespace base;
#include "glue/module.hh"
using namespace glue;
// Other modules:
#include "gfx/module.hh"
#include "input/module.hh"
#include "control/module.hh"
#include "physics/module.hh"
#include "view/module.hh"
#include "world/module.hh"
#include "hud/module.hh"
#if COMPILE_GUI
#include "gui/module.hh"
#endif
#include "sky/module.hh"
#include "sound/module.hh"
#include "game/module.hh"
#include "init/module.hh"

namespace init {

INTERN void SelfChecks( void );

/*****************************************************************************
 * Initialize modules (those applicable).
 *****************************************************************************/
void
Init( const InitArgs& initArgs )
{
    base::Init( initArgs );
    glue::Init( initArgs );
    gfx::Init( initArgs );
    world::Init( initArgs );
    physics::Init( initArgs );
    view::Init( initArgs );
	hud::Init( initArgs );
#if COMPILE_GUI
	gui::Init( initArgs );
#endif
    sky::Init( initArgs );
	sound::Init( initArgs );
    input::Init( initArgs );
    control::Init( initArgs );  // CONTROL_CONF depends on WORLD_CONF
	game::Init( initArgs );
	SelfChecks();
}

/*****************************************************************************
 * Start modules (those applicable).
 *****************************************************************************/
void
Start( void )
{
    base::Start();
    glue::Start();
    gfx::Start();
    physics::Start();
    view::Start();
    world::Start();
	hud::Start();
#if COMPILE_GUI
	gui::Start();
#endif
    sky::Start();
	sound::Start();
    input::Start();
    control::Start();
	game::Start();
}

/*****************************************************************************
 * Self-checks.
 *****************************************************************************/
void
SelfChecks( void )
{
#if DEBUG
	// Ensure the matrix indexs Xx etc are correct for OSG matrix.
 {
	Matrix m;
	m.makeIdentity();
	if ( m[Xx] != 1.0f or m[Yy] != 1.0f or m[Zz] != 1.0f
      or m[Xy] != 0.0f
      or m[Ox] != 0.0f or m[Oy] != 0.0f or m[Oz] != 0.0f )
	{
		throw Exception( "DEBUG: Matrix defs Xx etc are wrong!" );
	}
 }
	// Repeat but without makeIdentity().
 {
	Matrix m;
  //m.makeIdentity();
	if ( m[Xx] != 1.0f or m[Yy] != 1.0f or m[Zz] != 1.0f
      or m[Xy] != 0.0f
      or m[Ox] != 0.0f or m[Oy] != 0.0f or m[Oz] != 0.0f )
	{
		throw Exception( "DEBUG: Matrix defs Xx etc are wrong!" );
	}
 }
#endif // DEBUG
}

} // namespace init
