/*****************************************************************************
 * $LastChangedDate: 2008-05-04 16:23:01 -0400 (Sun, 04 May 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	This module contains bootstrap/kickstart routines.
 * @verbatim
 *
 * The top-level of the app:
 *
 * void
 * main( int argc, char** argv )
 * {
 *     init::Init( init::InitArgs(..) );
 *     init::Start();
 * }
 *
 * @endverbatim
 * @namespace init
 * @brief Kickstart routines.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef INIT_MODULE_HH
#define INIT_MODULE_HH 1

//------------------------------------------------------------------------------
// Coding convention for modules:
// - module.hh is the main header files that includes often-used header files.
// - Init() initializes the module (if applicable).
// - Start() starts the execution of a module (if applicable).

namespace init {

typedef base::InitArgs InitArgs;

void Init( const InitArgs& initArgs );

void Start( void );

} // namespace init

//------------------------------------------------------------------------------

#endif // INIT_MODULE_HH
