/*****************************************************************************
 * $LastChangedDate: 2008-05-04 16:23:01 -0400 (Sun, 04 May 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Linux Joystick class.
 * @remarks This is compiled on FreeBSD also.
 *          FreeBSD's devel/linux-js port emulates a Linux joystick driver.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#if OS_LINUX || OS_FREEBSD
#ifndef INPUT_JOYSTICK_LINUX_HH
#define INPUT_JOYSTICK_LINUX_HH 1

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/joystick.h>

namespace input {

////////////////////////////////////////////////////////////////////////////////
/// @brief Linux joystick class.
///
class JoystickLinux : public Joystick
{
PREVENT_COPYING(JoystickLinux)
friend class Joystick;  // for Joystick::GetInstance()
private:
	/// Axis numbers from Linux joystick driver event.
	enum
	{
		eLinuxAxis_LEFT_RIGHT = 0,
		eLinuxAxis_UP_DOWN	  = 1,
		eLinuxAxis_THIRD	  = 2,
	};

private:
			// Joystick is meant to be instantiated once.
			JoystickLinux( void );
			~JoystickLinux();

public:
	void	PrintStatus( void );

private:
	bool	Open( void );
	void	Close( void );
	bool	PollEvent( Event& event /*OUT*/ );

private:
	const string	mDeviceName;
	const string	mDeviceName2;	// alternate name
	int				mFiledesc;		// joystick device file
};

} // namespace input

#endif // INPUT_JOYSTICK_LINUX_HH
#endif // OS_LINUX || OS_FREEBSD
