/*****************************************************************************
 * $LastChangedDate: 2008-06-12 05:14:07 -0400 (Thu, 12 Jun 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	OSG-specific Keyboard class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#if COMPILE_OSG

#define INPUT_KEYBOARD_OSG_CC 1
#include "base/module.hh"
using namespace base;
#include "view/module.hh"
#include "view/window.hh"  // access to OSG Viewer to register keyboard handler
using namespace view;
#include "input/module.hh"
#include "input/keyboard.hh"
#include "input/keyboard_osg.hh"

namespace input {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////  KeyboardOSG  ////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
KeyboardOSG::KeyboardOSG( void )
{
ASSERT_RUN_ONCE;
	// NOP
}

KeyboardOSG::~KeyboardOSG()
{
	// NOP
}

/*****************************************************************************
 * Install/activate keyboard handlers.
 *****************************************************************************/
void
KeyboardOSG::Enable( void )
{
BEGIN_ONCE {
    // Register handler with OSG Viewer which can be obtained
    // from the Window singleton (in view module).
    Window::GetInstance().OsgSetKeyboardHandler( new KeyboardOSG::HandlerOSG );
} END_ONCE
}

/*****************************************************************************
 * Handles keyboard events sent from OSG.
 *****************************************************************************/
bool
KeyboardOSG::HandlerOSG::handle( const osgGA::GUIEventAdapter& eventOSG, osgGA::GUIActionAdapter& )
{
	// Note: OSG events are indicated by bits.

	// Also enqueue key release/up.
	if ( eventOSG.getEventType() & (osgGA::GUIEventAdapter::KEYDOWN
                                   |osgGA::GUIEventAdapter::KEYUP) )
    {
        // Translate OSG event to Keyboard::Event struct (don't confuse event structs).
        Keyboard::Event event;
        event.mKey     = eventOSG.getKey();
        event.mPress   = BOOL_CAST(eventOSG.getEventType() & osgGA::GUIEventAdapter::KEYDOWN);
        event.mShift   = BOOL_CAST(eventOSG.getModKeyMask() & (osgGA::GUIEventAdapter::MODKEY_LEFT_SHIFT | osgGA::GUIEventAdapter::MODKEY_RIGHT_SHIFT));
        event.mCtrl    = BOOL_CAST(eventOSG.getModKeyMask() & (osgGA::GUIEventAdapter::MODKEY_LEFT_CTRL  | osgGA::GUIEventAdapter::MODKEY_RIGHT_CTRL));
        event.mAlt     = BOOL_CAST(eventOSG.getModKeyMask() & (osgGA::GUIEventAdapter::MODKEY_LEFT_ALT   | osgGA::GUIEventAdapter::MODKEY_RIGHT_ALT));
        event.mSpecial = BOOL_CAST(event.mShift or event.mCtrl or event.mAlt);

		// OSG remaps Ctrl+letter as {1,..,26}.  Remap to {'a',..,'z'}.
		if ( event.mCtrl )
		{
			if ( event.mKey >= 1 and event.mKey <= 26 )
			{
				event.mKey += 96;
			}
		}

    	// Enqueue event into InputQueue.
    	InputQueue::GetInstance().EnqueueKeyEvent( event );

        return true;
    }
    else
    {
        return false;
    }
}

void
KeyboardOSG::HandlerOSG::accept( osgGA::GUIEventHandlerVisitor& v )
{
    v.visit( *this );
}

} // namespace input

#endif // COMPILE_OSG
