/*****************************************************************************
 * $LastChangedDate: 2008-05-03 17:36:09 -0400 (Sat, 03 May 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Input header file.
 * @namespace input
 * @brief input module
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef INPUT_MODULE_HH
#define INPUT_MODULE_HH 1

//------------------------------------------------------------------------------
// Coding convention for modules:
// - module.hh is the main header files that includes often-used header files.
// - Init() initializes the module (if applicable).
// - Start() starts the execution of a module (if applicable).

namespace input {

typedef base::InitArgs InitArgs;

void Init( const InitArgs& initArgs );

void Start( void );

} // namespace input

//------------------------------------------------------------------------------

#if COMPILE_GLUT
// [2008/02] Compiling GLUT is incompatible with OSG (OSG doesn't use/initialize it).
// GLUT joystick functions require glutInit() to be called prior, but OSG doesn't use GLUT at all.
#if GLUT_API_VERSION >= 4
#define COMPILE_GLUT_JOYSTICK 1
#endif
#endif // COMPILE_GLUT

#include "base/module.hh"
#include "base/time.hh"
using namespace base;
#include "types/module.hh"
using namespace types;
#include "input/keyboard.hh"
#include "input/joystick.hh"

// System-specific Keyboard class name.
#if COMPILE_OSG
#	define INPUT_KEYBOARD_CLASS KeyboardOSG
#	include "input/keyboard_osg.hh"
#elif COMPILE_GLUT_KEYBOARD
#	define INPUT_KEYBOARD_CLASS KeyboardGlut
#	include "input/keyboard_glut.hh"
#else
#	error
#endif

// System-specific Joystick classes.
// FreeBSD's devel/linux-js port emulates a Linux joystick driver.
// Joystick::GetInstance() is responsible, in part, for choosing which Joystick object.
#if OS_LINUX || OS_FREEBSD  // don't write if/else to compile every possible driver
#	include "input/joystick_linux.hh"
#endif
#if OS_WINDOWS
#	include "input/joystick_windows.hh"
#endif
#if COMPILE_GLUT_JOYSTICK  // not elif
#	include "input/joystick_glut.hh"
#endif
#include "input/joystick_nop.hh"  // always

// Input queue:
#include "input/input_queue.hh"

#endif // INPUT_MODULE_HH
