/*****************************************************************************
 * $LastChangedDate: 2008-10-06 10:35:00 -0400 (Mon, 06 Oct 2008) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "hud/module.hh"
#include "hud/hud_box.hh"
using namespace hud;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "HudBox"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================
// Need a function to check typesig because of macro/template limitations.
#if ! DOXYGEN
INTERN void
CheckTypesigHudBox( shptr<HudBox> obj )
{
    CHECK_TYPESIG(obj,TYPESIG_HUD_BOX);
}
#endif // ! DOXYGEN

/*****************************************************************************
 * Lua's garbage collector indirectly invokes this C++ destructor.
 *****************************************************************************/
INTERN int 
HudBox_Destroy( lua_State* L ) 
{ 
   try { 

		return DESTROY_USERDATA_SHPTR<HudBox>( L, CheckTypesigHudBox );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * HudBox:new()
 *****************************************************************************/
INTERN int 
HudBox_New( lua_State* L ) 
{ 
   try { 

		// Attach to the Hud.
		shptr<HudBox> hudBox = new HudBox;
		Hud::GetInstance().Attach( hudBox );
		return NEW_USERDATA_SHPTR<HudBox>( L, hudBox, BIND_METATABLE_NAME );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * HudBox:Set2D( x1, y1, x2, y2 ) Define box in 2D terms. 
 *****************************************************************************/
INTERN int 
HudBox_Set2D( lua_State* L ) 
{ 
   try { 

		const uint y2 = POP_INT( L );
		const uint x2 = POP_INT( L );
		const uint y1 = POP_INT( L );
		const uint x1 = POP_INT( L );
		shptr<HudBox> hudBox = POP_USERDATA_AS_SHPTR<HudBox>( L );
		CHECK_TYPESIG(hudBox,TYPESIG_HUD_BOX);
		hudBox->Set2D( Rect<int>( x1, y1, x2, y2 ) );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * HudBox:Set3D( vec3, width, height )
 * Define box in 3D terms.
 * It sets the rectangle in terms of a 3D position and 2D width,height.
 *****************************************************************************/
INTERN int 
HudBox_Set3D( lua_State* L ) 
{ 
   try { 

		const uint        height2 = POP_INT( L );
		const uint        width2  = POP_INT( L );
		const WorldVertex pos3    = POP_VECTOR3<WorldVertex>( L );
		shptr<HudBox> hudBox = POP_USERDATA_AS_SHPTR<HudBox>( L );
		CHECK_TYPESIG(hudBox,TYPESIG_HUD_BOX);
		hudBox->Set3D( pos3, width2, height2 );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * HudBox:SetColor( r, g, b, a )  (fp,fp,fp,fp)
 *****************************************************************************/
INTERN int 
HudBox_SetColor( lua_State* L ) 
{ 
   try { 

		const RGBA color = POP_RGBA( L );
		shptr<HudBox> hudBox = POP_USERDATA_AS_SHPTR<HudBox>( L );
		CHECK_TYPESIG( hudBox, TYPESIG_HUD_BOX );
		hudBox->SetColor( color );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  { "new", HudBox_New },
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  { "__gc", HudBox_Destroy },
  { "Set2D", HudBox_Set2D },
  { "Set3D", HudBox_Set3D },
  { "SetColor", HudBox_SetColor },
  { NULL, NULL }
};

void
RegisterHudBox( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
