/*****************************************************************************
 * $LastChangedDate: 2008-10-14 21:02:53 -0400 (Tue, 14 Oct 2008) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks  * sim.Joystick abstracts control::Joystick.
 * This binding is unusual in that RegisterJoystick() also registers
 * a listener for the joystick button event to propagate to Lua.
 * No "dequeue axis" method is exported to Lua since the control module
 * handles axis events.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "control/module.hh"
#include "control/joystick.hh"
using namespace control;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Joystick"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

#if ! DOXYGEN
////////////////////////////////////////////////////////////////////////////////
/// @brief Joystick button handler that propagates to Lua script.
/// This queues and dequeues events.
///
class ListenerJoystickButton : public Event<input::Joystick::Event>::ListenerFunctor
{
public:
	ListenerJoystickButton( void ) : mEvents() { }

	// Get joystick event.  Caller should ignore axis event (just in case).
	input::Joystick::Event
	GetJoyEvent( void )
	{
		input::Joystick::Event event;  // invalid
		if ( not mEvents.empty() )
		{
			event = mEvents.front();  // dequeue
			mEvents.pop_front();
		}
		return event;
	}

private:
	void operator()( input::Joystick::Event event )
	{
	ASSERT( mEvents.size() < 50000 );
		mEvents.push_back( event );  // enqueue
	}

private:
	deque<input::Joystick::Event> mEvents;
};
#endif // ! DOXYGEN

// The joystick button handler.
INTERN shptr<ListenerJoystickButton> sListenerJoystickButton = new ListenerJoystickButton;


//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * table|nil Joystick:DequeueButton()
 * Returns table { buttonNum=int, buttonVal=bool }
 *****************************************************************************/
INTERN int 
Joystick_DequeueButton( lua_State* L ) 
{ 
   try { 

		// Despite the event struct also containing axis members, ignore them.
		// Only propagate the button members to Lua since the control module handles axis events.
		input::Joystick::Event joyEvent = sListenerJoystickButton->GetJoyEvent();
		if ( joyEvent.mButton != input::Joystick::eButton_NONE )
		{
			lua_newtable( L );
			SET_MEMBER_AS_INT(  L, "buttonNum", joyEvent.mButton );
			SET_MEMBER_AS_BOOL( L, "buttonVal", joyEvent.mButtonVal );
		}
		else
		{
			lua_pushnil( L );  // no button event
		}
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * int16 Joystick:GetAxisValue( axis )
 *****************************************************************************/
INTERN int 
Joystick_GetAxisValue( lua_State* L ) 
{ 
   try { 

		const int axis = POP_INT( L );
		const int axisVal = control::Joystick::GetInstance().GetAxisValue( axis );
		lua_pushnumber( L, axisVal );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  // new,__gc omitted as Lua cannot instantiate this C++ class.
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  // __gc omitted as Lua cannot instantiate this C++ class.
  { "DequeueButton", Joystick_DequeueButton },
  { "GetAxisValue", Joystick_GetAxisValue },
  { NULL, NULL }
};

void
RegisterJoystick( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );

	// Listen for joystick button events.
	control::Joystick::GetInstance().mEventButton.Listen( sListenerJoystickButton.PTR() );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
