-- $LastChangedDate: 2008-12-04 11:05:12 -0500 (Thu, 04 Dec 2008) $
-- Macros written in Lua to expand C++ code for bindings between C++/Lua.

bindTable = {

class = "Program",
typesig = nil,
luaCanInstantiate = false,

headers =
[[
#include "base/module.hh"
using namespace base;
#include "program/module.hh"
using namespace program;
]],

func_IfDebug = {
    comments = "bool Program:IfDebug()",
    code = [[
#if DEBUG
		lua_pushboolean( L, true );
#else
		lua_pushboolean( L, false );
#endif
		return 1;  // number of pushed values
    ]]
},

func_GetMemUsed = {
    comments = "int Program:GetMemUsed()",
    code = [[
#if DEBUG
		lua_pushnumber( L, MemUsed() );
#else
		lua_pushnumber( L, 0 );  // N/A in non-DEBUG
#endif
		return 1;  // number of pushed values
    ]]
},

func_GetScriptDir = {
    comments = "string Program:GetScriptDir()",
    code = [[
		lua_pushstring( L, GET_LUA().GetScriptDir().c_str() );
		return 1;  // number of pushed values
    ]]
},

func_GetElapsedTime = {
    comments = "milliseconds Program:GetElapsedTime()",
    code = [[
		lua_pushnumber( L, base::Timer::GetInstance().GetElapsedTime() );
		return 1;  // number of pushed values
    ]]
},

func_Reset = {
    comments = "Program:Reset()",
    code = [[
		gProgram.Reset();
    ]]
},

func_Exit = {
    comments = "Program:Exit( int )",
    code = [[
		EXIT( POP_INT( L ) );
    ]]
},

func_SetPause = {
    comments = "Program:SetPause( bool )",
    code = [[
		gProgram.SetPause( POP_BOOL( L ) );
    ]]
},

func_IfPaused = {
    comments = "bool Program:IfPaused()",
    code = [[
		lua_pushboolean( L, gProgram.IfPaused() );
		return 1;  // number of pushed values
    ]]
},

func_GetTickFreq = {
    comments = "milliseconds Program:GetTickFreq()\n * Frequency of program's timer-tick (its pulse).",
    code = [[
		lua_pushnumber( L, PROGRAM_CONF.mTimerTickFreq );
		return 1;  // number of pushed values
    ]]
},

func_DequeueCommand = {
    comments = "string Program:DequeueCommand()\n * Returns next command or nil.",
    code = [[
		const string cmd = gProgram.DequeueCommand();
		if ( cmd.empty() )
			lua_pushnil( L );
		else
			lua_pushstring( L, cmd.c_str() );
		return 1;  // number of pushed values
    ]]
},

}
