/*****************************************************************************
 * $LastChangedDate: 2008-04-10 19:14:32 -0400 (Thu, 10 Apr 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Vector/vertex types.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef MATH_VERTEX_HH
#define MATH_VERTEX_HH 1

#include <osg/Vec2>
#include <osg/Vec2f>
#include <osg/Vec2d>
#include <osg/Vec3>
#include <osg/Vec3f>
#include <osg/Vec3d>

/// @class math::Vector2
/// @class math::Vector3
/// @class math::LocalVertex
/// @class math::WorldVertex
/// @class math::EyeVertex
#define Vector2      osg::Vec2f
#define Vector3      osg::Vec3f
#define LocalVertex  osg::Vec3f
#define WorldVertex  osg::Vec3f
#define EyeVertex    osg::Vec3f

//==============================================================================
// KLUDGE: To compile operator<<(osg::Vec3), place inside the osg namespace.

namespace osg {

/*****************************************************************************
 * operator<<(Vector2)
 *****************************************************************************/
// Vector2.
INLINE ostream&
operator<<( ostream& strm, const Vector2& v )
{
	ios::fmtflags savedFlags = strm.flags();
    strm << "Vector2: ("
         << OMANIP_FIELD(8) << v[math::XX] << ','
         << OMANIP_FIELD(8) << v[math::YY] << ')';
	strm.flags(savedFlags);
	return strm;
}

/*****************************************************************************
 * operator<<(Vector3)
 *****************************************************************************/
INLINE ostream&
operator<<( ostream& strm, const Vector3& v )
{
	ios::fmtflags savedFlags = strm.flags();
    strm << "Vector3: ("
         << OMANIP_FIELD(8) << v[math::XX] << ','
         << OMANIP_FIELD(8) << v[math::YY] << ','
         << OMANIP_FIELD(8) << v[math::ZZ] << ')';
	strm.flags(savedFlags);
	return strm;
}

} // namespace osg

//==============================================================================

#endif // MATH_VERTEX_HH
