/*****************************************************************************
 * $LastChangedDate: 2008-08-29 01:40:45 -0400 (Fri, 29 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Module configuration.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef PROGRAM_CONF_HH
#define PROGRAM_CONF_HH 1

#include "base/module.hh"
#include "base/conf_txt.hh"
using namespace base;
#include "object/module.hh"
#include "object/rotation_list.hh"
using namespace object;
#include "world/module.hh"
using namespace world;

namespace program {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
	////////////////////////////////////////////////////////////////////////////
	/// @brief Template Method to parse fields in .conf file.
	///
	class ModuleConfTxt : public base::ConfTxt
	{
	public:
	    void ProcessField( string& field, ifstream& conf );
	};

public:
			Conf( void );
	void	Init( int argc, char** argv, const string& confFilename );

private:
	void	ParseCmdlineArgs( int argc, char** argv );
	void	ParseCmdlineArgs2( int argc, char** argv );
	void	ReadConfTxt( const string& confTxtFilename );

public:
	bool				mValid;     ///< true if Conf was init/read

	// Cmd-line args:
	bool				mPrintJoystick;

	// .conf:
	Milliseconds		mTimerTickFreq;					///< app's tick
	RotationList		mAircraftRotationList;			///< initial orientation
	string				mCarrierModelName;
	RotationList		mCarrierRotationList;			///< initial orientation
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if PROGRAM_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// PROGRAM_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(program::global::gConf.mValid);
                               return program::global::gConf; }
#define PROGRAM_CONF program::GetConf()
#else
#define PROGRAM_CONF program::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace program

#endif // PROGRAM_CONF_HH
