/*****************************************************************************
 * $LastChangedDate: 2008-12-02 12:16:59 -0500 (Tue, 02 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Top-level program.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef PROGRAM_PROGRAM_HH
#define PROGRAM_PROGRAM_HH 1

#include "control/module.hh"
using namespace control;
#include "view/module.hh"
using namespace view;

namespace program {

class Carrier;

////////////////////////////////////////////////////////////////////////////////
/// @brief The top-level of the program.
///
/// Control and Lua define the "current" Craft.
///
class Program
{
public:
                        Program( void );
                        ~Program();

// Program control:
	void                Run( int argc, char** argv );
	void				Reset( void );
	void				SetPause( bool pause_ );  // pause() is a syscall
	bool				IfPaused( void );

// Simulator's command queue:
	void				EnqueueCommand( const string& cmd );  // command queue
	string				DequeueCommand( void );

private:
	void				Tick( const Milliseconds millisecElapsed );
#if DEBUG
	void				SelfChecks( void );
#endif

private:
	/// @brief Timer-tick.
	///
    CLASS_TICK_FUNCTOR_BASIC

// Data:
private:
	bool                    mPause;
	SafePtr<Window>         mWindow;
	SafePtr<Control>        mControl;
	typedef list<string> CommandQueue;
	CommandQueue			mCommandQueue;

};

} // namespace program

#endif // PROGRAM_PROGRAM_HH
