/*****************************************************************************
 * $LastChangedDate: 2008-08-21 08:48:11 -0400 (Thu, 21 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Module configuration.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SKY_CONF_HH
#define SKY_CONF_HH 1

#include <osg/Fog>
#include "base/conf_txt.hh"
#include "types/rgba.hh"
using namespace types;
#include "physics/module.hh"
using namespace physics;

namespace sky {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
public:
	struct CloudConf
	{
		CloudConf( void )
		: mCloudCnt(0), mCloudMinAlt(0), mCloudMaxAlt(0),
		  mCloudMinWidth(0), mCloudMaxWidth(0) { }
		uint	mCloudCnt;			///< amount of clouds
		Meter	mCloudMinAlt;		///< random range
		Meter	mCloudMaxAlt;
		Meter	mCloudMinWidth;		///< random range
		Meter	mCloudMaxWidth;
	};

private:
	////////////////////////////////////////////////////////////////////////////
	/// @brief Template Method to parse fields in .conf file.
	///
	class ModuleConfTxt : public base::ConfTxt
	{
	public:
	    void ProcessField( string& field, ifstream& conf );
	    void ProcessField( string& field, ifstream& conf, CloudConf& cloudConf );
	};

public:
			Conf( void );
	void	Init( int argc, char** argv, const string& confTxtFilename );

private:
	void	ParseCmdlineArgs( int argc, char** argv );
	void	ParseCmdlineArgs2( int argc, char** argv );
	void	ReadConfTxt( const string& confTxtFilename );

public:
	bool            mValid;     ///< true if Conf was init/read

	// Cmd-line args:
    // (none)

	// .conf:
	RGBA			mSkyColor;
	fpx				mSkyFogDensity;		///< fpx, sky needs lower fog density
	uint			mSkyDomeSliceCnt;
	bool			mCloudsEnabled;
	CloudConf		mCloudConfFlat;
	CloudConf		mCloudConfSphere;
	uint			mStarCnt;			///< amount of stars
	bool			mLightEnabled;		///< disabled by -no-light
	bool			mFogEnabled;
	RGBA			mFogColor;
    osg::Fog::Mode	mFogMode;			///< osg::Fog::EXP (GL_EXP) etc
	fpx				mFogDensity;		///< fpx, general fog density
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if SKY_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// SKY_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(sky::global::gConf.mValid);
                               return sky::global::gConf; }
#define SKY_CONF sky::GetConf()
#else
#define SKY_CONF sky::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace sky

#endif // SKY_CONF_HH
