/*****************************************************************************
 * $LastChangedDate: 2008-12-06 20:04:25 -0500 (Sat, 06 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Sky model (Singleton).
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SKY_SKY_HH
#define SKY_SKY_HH 1

#include "graph/module.hh"
#include "graph/light.hh"
using namespace graph;

namespace sky {

#define GET_SKY()		(Sky::GetInstance())		// returns ref
#define GET_SKY_DOME()	(GET_SKY().GetSkyDome())	// returns ref also

// This header is the public interface, so avoid including implementation headers.
class SkyDome;
class CloudParticleSystem;
class Sun;
class Moon;
class Stars;

////////////////////////////////////////////////////////////////////////////////
/// @brief Sky model (Singleton).
///
class Sky
{
PREVENT_COPYING(Sky)
private:
						Sky( void );
						~Sky();

public:
    CLASS_METHOD Sky& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new Sky;
		return *msInstance;
	}

	void				Start( const fp hour );
	void				SetHour( const fp hour );  // fp to allow gradations of sun's movement
	shptr<Light>		GetLight( void );

public:
// LIMITED TO SKY-RELATED CLASSES:
	BoxVolume			GetVolume( void );
	SkyDome&			GetSkyDome( void );
	Sun&				GetSun( void );

private:
	void				InitLightingAndFog( const fp hour );
	pair<Degree,Degree> ComputeLatLonFromHour( const fp hour );  // compute sun's position

private:
	CLASS_METHOD Sky*			msInstance;				///< Singleton
	shptr<SkyDome>				mSkyDome;				///< sky dome
	shptr<CloudParticleSystem>	mCloudParticleSystem;	///< clouds
	shptr<Sun>					mSun;					///< position of sun in sky
	shptr<Moon>					mMoon;					///< Object-derivative
	shptr<Stars>				mStars;					///< the Stars (Object-derivative)
	shptr<Light>				mLight;
};

} // namespace sky

#endif // SKY_SKY_HH
