/*****************************************************************************
 * $LastChangedDate: 2008-11-08 10:57:53 -0500 (Sat, 08 Nov 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Sound class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#if SOUND_PLIB
#ifndef SOUND_SOUND_PLIB_HH
#define SOUND_SOUND_PLIB_HH 1

#include "base/timer.hh"

class slScheduler;  // alternative to including plib headers here
class smMixer;
class slSample;

namespace sound {

////////////////////////////////////////////////////////////////////////////////
/// @brief Sound class based on PLIB.
///
/// Sounds are identified/keyed by filename.
///
/// Gory details of PLIB code is better structured by making Sample
/// mostly private to Sound, but letting Sample have full access to Sound.
///
class SoundPlib : public SoundAbstract
{
PREVENT_COPYING(SoundPlib)
private:
	SoundPlib( void );
	~SoundPlib();

private:
	////////////////////////////////////////////////////////////////////////////
	/// @brief Defines a sound sample.
	///
	class Sample : public Shared
	{
	public:
					Sample( const string& filename, const fp volume = DEFAULT_VOLUME );
					~Sample();
		void		Play( const bool loop, const fp volume );
		void		Stop( void );
		bool		IfPlaying( void );
		void		SetVolume( const fp volume );
		fp			GetVolume( void );

	private:
		fp			mVolume;		///< current volume (PLIB alters waveform data)
		bool		mLoop;			///< whether to repeat sample
		slSample*	mSLSample;		///< PLIB audio object
		string		mBuf;			///< stored for changing volume
	public:
		DECL_TYPESIG(TYPESIG_SOUND_SAMPLE)
	};

//..............................................................................

public:
	CLASS_METHOD SoundPlib&	GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new SoundPlib;
		return *msInstance;
	}
	CLASS_METHOD void		Disable( void );  // don't use with GetInstance()
	bool					IfPlaying( const string& filename );
	bool					IfPlaying( const string& filename, const fp volume );
	// Play() is a NOP if sample is already playing with the same settings.
	bool					Play( const string& filename, const bool loop, const fp volume = DEFAULT_VOLUME );
	void					Stop( const string& filename );
	void					StopAll( void );
	void					SetVolume( const string& filename, const fp volume );

private:
	void					Pulse( void );
	void					EnqueueZombieSample( slSample* zombie );

private:
	/// @brief Timer-tick.
	///
    CLASS_TICK_FUNCTOR_BASIC

private:
	typedef map<string,shptr<Sample> > SampleMap;

	CLASS_VAR SoundPlib*	msInstance;
	CLASS_VAR bool			msEnabled;			///< for permanently disabling sound
	SampleMap				mSampleMap;			///< to load a sound file once but play it multiple times
	slScheduler*			mSched;				///< PLIB
	smMixer*				mMixer;				///< PLIB
	set<slSample*>			mZombieSamples;		///< samples that were discarded after changing volume (yuck)
};

} // namespace sound

#endif // SOUND_SOUND_PLIB_HH
#endif // SOUND_PLIB
