/******************************************************************************
 * $LastChangedDate: 2008-09-01 14:12:18 -0400 (Mon, 01 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	RGBA class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 ******************************************************************************/

#define TYPES_RGBA_CC 1
#include "base/module.hh"
using namespace base;
#include "types/module.hh"
#include "types/rgba.hh"

namespace types {

/*****************************************************************************
 * operator<<(RGBA)
 *****************************************************************************/
ostream&
operator<<( ostream& strm, const RGBA rgba )
{
	ios::fmtflags savedFlags = strm.flags();
    strm << '('
         << OMANIP_FIELD(3) << uint(rgba[RR]) << ','  // print numbers, not chars
         << OMANIP_FIELD(3) << uint(rgba[GG]) << ','  // (uint8 is uchar)
         << OMANIP_FIELD(3) << uint(rgba[BB]) << ','
         << OMANIP_FIELD(3) << uint(rgba[AA]) << ')';
	strm.flags(savedFlags);
	return strm;
}

/*****************************************************************************
 * Interpolate between two RGBAs.
 * @param	rgba0
 * @param	rgba1
 * @param	factor
 *          0.0 : rgba0
 *			0.5 : interpolate at mid-point.
 *          1.0 : rgba1
 *****************************************************************************/
RGBA
Interpolate( const RGBA rgba0, const RGBA rgba1, const fp factor )
{
ASSERT( FP_GE(factor,0.0f) and FP_LE(factor,1.0f) );
	return RGBA( uint8( int(rgba0[RR]) + (int(rgba1[RR]) - int(rgba0[RR])) * factor ),
                 uint8( int(rgba0[GG]) + (int(rgba1[GG]) - int(rgba0[GG])) * factor ),
                 uint8( int(rgba0[BB]) + (int(rgba1[BB]) - int(rgba0[BB])) * factor ),
                 uint8( int(rgba0[AA]) + (int(rgba1[AA]) - int(rgba0[AA])) * factor ) );
}

} // namespace types
