/*****************************************************************************
 * $LastChangedDate: 2008-12-05 10:58:31 -0500 (Fri, 05 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Main header file.
 * @namespace view
 * @brief view module (View, Viewpoint, Window).
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef VIEW_MODULE_HH
#define VIEW_MODULE_HH 1

//------------------------------------------------------------------------------
// Coding convention for modules:
// - module.hh is the main header files that includes often-used header files.
// - Init() initializes the module (if applicable).
// - Start() starts the execution of a module (if applicable).

namespace view {

typedef base::InitArgs InitArgs;

void Init( const InitArgs& initArgs );

void Start( void );

} // namespace view

//------------------------------------------------------------------------------

#include <osg/GraphicsContext>
#include <osgViewer/Viewer>
#include <osgViewer/ViewerEventHandlers>  // for OSG statistics display

#include "base/time.hh"
#include "base/event.hh"
using namespace base;
#include "math/module.hh"
#include "math/matrix.hh"
using namespace math;
#include "gfx/module.hh"
#include "gfx/osg.hh"
using namespace gfx;
#include "types/module.hh"
#include "types/rgba.hh"
using namespace types;
#if DEBUG
#include "view/typesig_defs.hh"
#endif
#include "view/viewpoint.hh"
#include "view/view.hh"
#include "view/chase_viewpoint.hh"
#include "view/window.hh"
#include "view/events.hh"

#endif // VIEW_MODULE_HH
