/*****************************************************************************
 * $LastChangedDate: 2008-11-10 19:53:52 -0500 (Mon, 10 Nov 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Viewpoint class.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define VIEW_VIEWPOINT_CC 1
#include "base/module.hh"
using namespace base;
#include "control/module.hh"
using namespace control;
#include "world/module.hh"
using namespace world;
#include "view/module.hh"
#include "view/conf.hh"
#include "view/viewpoint.hh"

namespace view {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  Viewpoint  //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
Viewpoint::Viewpoint( void )
:	mOtwMode(eOtwMode_OTW_FRONT)
{
	SET_TYPESIG(this,TYPESIG_VIEWPOINT);
}

Viewpoint::~Viewpoint()
{
	INVALIDATE_TYPESIG(this,TYPESIG_VIEWPOINT);
}

/*****************************************************************************
 * Computes view-matrix every frame.
 * A View object is composed with a Viewpoint object.
 *****************************************************************************/
Matrix
Viewpoint::ComputeViewMatrix( shptr<View> view )
{
CHECK_TYPESIG(view,TYPESIG_VIEW);

	// Compute an eye matrix from transposing the Craft's matrix.
	Matrix m = GET_CURRENT_CRAFT()->GetMatrix();

	// Mitigate frustum clipping when at low altitudes (carrier deck)
	// by adjusting the position (mOtwViewpointOffset is a raw world coordinate).
	if ( GET_WORLD().IfFlat() )  // N/A to OSSIM
	{
		MatrixTranslateLocal( m, YY, VIEW_CONF.mOtwViewpointOffset );
	}

	switch ( mOtwMode )
	{
		case eOtwMode_OTW_FRONT: default:
			// NOP
		break;

		case eOtwMode_OTW_REAR:
		{
			MatrixRotateLocal( m, ZZ, RADIAN_180 );
			MatrixRotateLocal( m, YY, RADIAN_180 );
			MatrixRotateLocal( m, ZZ, RADIAN_180 );
		}
		break;

		case eOtwMode_OTW_LEFT:
		{
			MatrixRotateLocal( m, YY, RADIAN_90 );
		}
		break;

		case eOtwMode_OTW_RIGHT:
		{
			MatrixRotateLocal( m, YY, -RADIAN_90 );
		}
		break;
	}

	return MatrixInvert( m );
}

} // namespace view
