/*****************************************************************************
 * $LastChangedDate: 2008-08-22 23:47:33 -0400 (Fri, 22 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Viewpoint class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef VIEW_VIEWPOINT_HH
#define VIEW_VIEWPOINT_HH 1

namespace view {

class View;

////////////////////////////////////////////////////////////////////////////////
/// @brief Functor that computes view matrix every frame.
///
/// Logical views are composed with a View and a Viewpoint object.
/// Viewpoint is a functor containing an algorithm to compute the view matrix
/// which is called before rendering a new frame.
/// This composition can be changed dynamically using View::SetViewpoint()
/// to allow switching to different view modes (first-person, chase-plane, etc).
/// Despite the view appearing to change, View keeps the same underlying osg::Camera object.
///
/// Viewpoint is capable of controlling first-person and chase-plane views
///	by converting the player's matrix into an eye matrix.
/// View/Viewpoint supercedes EyeCraft and ChasePlane of old versions.
///
class Viewpoint : public Shared
{
PREVENT_COPYING(Viewpoint);
public:
	/// @brief Out-The-Window view.
	enum eOtwMode
	{
		// (update Lua)
		eOtwMode_OTW_NONE,  // an example of no OTW is chase-view
		eOtwMode_OTW_FRONT,
		eOtwMode_OTW_REAR,
		eOtwMode_OTW_LEFT,
		eOtwMode_OTW_RIGHT,
	};

						Viewpoint( void );
	virtual				~Viewpoint();
	virtual Matrix		ComputeViewMatrix( shptr<View> view );
	virtual void		SetOtwMode( const eOtwMode otwMode ) { mOtwMode = otwMode; }
	virtual eOtwMode	GetOtwMode( void ) const { return mOtwMode; }

private:
	eOtwMode		mOtwMode;	///< OTW front,rear,left,right (or none)
public:
	DECL_TYPESIG(TYPESIG_VIEWPOINT);
};

} // namespace view

#endif // VIEW_VIEWPOINT_HH
