/*****************************************************************************
 * $LastChangedDate: 2008-09-13 22:52:35 -0400 (Sat, 13 Sep 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Object to contain group nodes sorted by graphical state.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_STATE_SORTER_HH
#define WORLD_STATE_SORTER_HH 1

namespace world {

/// @brief Bits for graphical states of nodes.
typedef uint64 StateBits;

/// These are somewhat app-specific.  Can't use enum which might be 32-bits.
CLASS_CONST uint64 STATE_BIT_DEFAULT		= 0;
CLASS_CONST uint64 STATE_BIT_NO_LIGHTING	= 1 << 0;
CLASS_CONST uint64 STATE_BIT_NO_FOG			= 1 << 1;
CLASS_CONST uint64 STATE_BIT_BLEND			= 1 << 2;
CLASS_CONST uint64 STATE_BIT_TRANSPARENT	= 1 << 3;
CLASS_CONST uint64 STATE_BIT_NO_DEPTH_TEST	= 1 << 4;

/// Shift/mask constants.  Mask values are at UNSHIFTED position.
CLASS_CONST uint64 STATE_BIT_MASK_ID		= 0xfff;		// 0xfff=4096
CLASS_CONST uint   STATE_BIT_SHIFT_ID		= 64 - 3*4;		// 3*4 is bits in 0xfff

////////////////////////////////////////////////////////////////////////////////
/// @brief Object to contain group nodes sorted by graphical state.
///
class StateSorter : public Shared
{
PREVENT_COPYING(StateSorter)
public:
							StateSorter( void );
							~StateSorter();
	RefPtr<GroupNode>		GetSortedGroup( RefPtr<GroupNode>     parent,
                                            const StateBits       stateBits,
                                            RefPtr<osg::StateSet> stateSet = NULL );
	CLASS_METHOD StateBits	AllocateStateBitsId( void );

private:
	typedef map<StateBits,RefPtr<GroupNode> > SortedGroup;
	CLASS_VAR StateBits		msId;				///< unique ID representing any combination of graphical states
	SortedGroup				mSortedGroups;		///< group nodes that sort-state (elements themselves aren't sorted of course)
};

} // namespace world

#endif // WORLD_STATE_SORTER_HH
