/*****************************************************************************
 * $LastChangedDate: 2008-12-07 22:48:41 -0500 (Sun, 07 Dec 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	World class.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_WORLD_HH
#define WORLD_WORLD_HH 1

#include "graph/module.hh"
using namespace graph;
#include "object/module.hh"
#include "object/events.hh"
using namespace object;
#include "world/state_sorter.hh"

namespace world {

#define GET_WORLD() (gWorld)	// world/globals.hh, gWorld is a macro
#define ASSERT_WORLD()			// NOP

////////////////////////////////////////////////////////////////////////////////
/// @brief World class is a container of 3D objects.
///
/// The representation of the simulated world is factored
/// into graphical and geodesic classes: Geo and World.
///
/// World functions as a container of Objects.
/// An Object ctor can ensure it has a reference by calling World::Attach(object)
/// (else, it the ctor passed itself shptr<Object> then premature deletion would occur).
///
class World
{
public:						World( void );
public:						~World();
public: void                Make( void );

// Interface:
public:	void				Attach( shptr<Object> object,
                                    const StateBits stateBits = STATE_BIT_DEFAULT,
                                    const RefPtr<osg::StateSet> stateSet = new osg::StateSet );
public:	void				Attach( RefPtr<Node> node,  // LIMITED-USE
                                    const StateBits stateBits = STATE_BIT_DEFAULT,
                                    const RefPtr<osg::StateSet> stateSet = new osg::StateSet );
public:	void				Detach( shptr<Object> object );
public:	void				Detach( RefPtr<Node> node );  // LIMITED-USE
public: void				OptimizeSceneGraph( void );  // to be called after loading objects
public: bool				IfFlat( void );
public: WorldVertex         GetPosition( void ) { return WorldVertex(0,0,0); }  // center
public: fp                  GetRadius( void );
public: bool                SetCompression( const bool enable );
public: fpx					GetCompressionScale( void );
public: void                RotateTangential( Object& object, const SphereVertex& sv );
public: const Matrix&		GetTangentMatrix( const SphereVertex& sv );
public: WorldVertex         ComputeUpVector( const SphereVertex& sv );
public: WorldVertex         ComputeUpVector( Object& object );
public: BoxVolume			GetVolume( void );
public: bool                IfLowAltitude( const WorldVertex& pos );

// Orientation:
public:  CLASS_CONST int ALT_UP_DIR   = 1;
public:  CLASS_CONST int ALT_DOWN_DIR = -1;
public:  bool               IfDownward( fp n ) { return n < 0; }
public:  bool               IfUpward( fp n )   { return n >= 0; }

private:
	CLASS_DESTROY_OBJECT_LISTENER( World, mWorld )

private:
    typedef set<shptr<Object> > Objects;
  //shptr<SceneGraph>		mSceneGraph;		///< Window now contains SceneGraph
	RefPtr<GroupNode>		mRootGroup;
	shptr<StateSorter>		mStateSorter;		///< state-sorting
    Objects                 mObjects;           ///< some Object ctors need to attach to hold a reference
};

} // namespace world

#endif // WORLD_WORLD_HH
