/*****************************************************************************
 * $LastChangedDate: 2008-04-22 20:27:04 -0400 (Tue, 22 Apr 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test of ConfTxt class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "base/module.hh"
#include "base/conf_txt.hh"
#include "base/file.hh"
using namespace base;

INTERN const string fname = "/tmp/base_conf_txt_test.txt";

class ConfTxtTest : public ConfTxt
{
public:
	ConfTxtTest( void )
	:	mLineCnt(0),
		mWindowWidth(),
		mWindowHeight()
	{
	}

private:
	virtual void ProcessField( string& field, ifstream& config )
	{
		if ( field == "WindowWidth" )
		{
			config >> mWindowWidth;
		}
		else if ( field == "WindowHeight" )
		{
			config >> mWindowHeight;
		}
		else
		{
			Fail();
		}
	}

public:
	int		mLineCnt;
	string	mWindowWidth;	// "800"
	string	mWindowHeight;	// "600"
};

/*****************************************************************************
 * Test.
 *****************************************************************************/
bool
Test1( void )
{
	// ------------------------------------------------------------------------
	// First series of tests are for when no configuration file exists yet.
	// ------------------------------------------------------------------------

	RemoveFile( fname );  // ensure configuration file doesn't exist

	ConfTxtTest conf;
	bool exception = false;

	// Test that the error of empty filename causes throwing an exception.
	exception = false;
	try
	{
		conf.Read( "" );
	}
	catch (...)
	{
		// ok
		exception = true;
	}
	if ( not exception )
	{
		CDEBUG << "FAIL: empty filename \n";
		return Fail();
	}

	return true;
}

/*****************************************************************************
 * Test reading/parsing lines of textual configuration.
 *****************************************************************************/
bool
Test2( void )
{
	RemoveFile( fname );  // ensure configuration file doesn't exist

	// Create a data file.
	fstream fs;
//	fs.open( fname.c_str(), ios::in|ios::out|ios::binary );  // wrong
	fs.open( fname.c_str(), ios::in|ios::out|ios::binary|ios::trunc );
	fs << "WindowWidth   800" << endl
       << "WindowHeight  600" << endl;
	if ( (not fs.good()) or (not fs.is_open()) )
	{
		return Fail();
	}

	// Read data file.
	ConfTxtTest conf;
	try
	{
		conf.Read( fname );
	}
	catch (...)
	{
		return Fail();
	}

	if ( conf.mWindowWidth == "800"
     and conf.mWindowHeight == "600" )
	{
		return true;
	}
	else
	{
		return Fail();
	}
}


/*****************************************************************************
 * main.
 *****************************************************************************/
int
main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;
	int res = 1;
	if ( Test1() and Test2() )
	{
		CDEBUG << "Test PASSED.\n";
		RemoveFile( fname );  // remove if test passed, else leave
		res = 0;
	}
	else
	{
		CDEBUG << "Test FAILED!\n";
		res = 1;
	}
	TEST_MAIN_END;
	exit( res );
}
