/*****************************************************************************
 * $LastChangedDate: 2008-05-03 21:56:42 -0400 (Sat, 03 May 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test of Seconds etc classes.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "base/module.hh"
#include "base/time.hh"
using namespace base;

/*****************************************************************************
 * Test
 *****************************************************************************/
bool
Test( void )
{
	{
	Milliseconds millsec( 1000 );
	Microseconds usec( millsec );
	if ( usec.mMicroseconds != MILLION ) return Fail();
	}

	{
	Microseconds usec( 2000000 );
	if ( usec.mMicroseconds != 2000000 ) return Fail();
	}

	{
	Microseconds usec( Seconds(6) );
	if ( usec.mMicroseconds != 6*MILLION ) return Fail();
	}

	{
	Milliseconds usec( 12345678 );
	if ( usec.mMilliseconds != 12345678 ) return Fail();
	}

	{
	Milliseconds usec( Seconds(4) );
	if ( usec.mMilliseconds != 4000 ) return Fail();
	}

	{
	Microseconds usecs( MILLION );
	if ( int(usecs) != MILLION ) return Fail();
	}

	{
	Milliseconds millsecs( THOUSAND );
	if ( int(millsecs) != THOUSAND ) return Fail();
	}

	{
	Microseconds usec( Milliseconds(1000) );
	if ( int(usec) != MILLION ) return Fail();
	}

	{
	Microseconds usec( Seconds(1) );
	if ( int(usec) != MILLION ) return Fail();
	}

	{
	Microseconds usec(1); int i = 1;
	usec = usec + 100; i = i + 100;
	usec = usec - 20;  i = i - 20;
	usec = usec * 10;  i = i * 10;
	usec = usec / 2;   i = i / 2;
	if ( int(usec) != i ) return Fail();
	}

	{
	Microseconds a(99);
	Microseconds b(100);
	Microseconds c(100);
	if ( not (a < b) ) return Fail();
	if ( not (b > a) ) return Fail();
	if ( not (b >= c) ) return Fail();
	if ( not (b <= c) ) return Fail();
	}

	return true;
}

int main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;
	int res = 1;
	if ( Test() )
	{
		CDEBUG << "Test PASSED. \n";
		res = 0;
	}
	else
	{
		CDEBUG << "Test FAILED! \n";
		res = 1;
	}
	TEST_MAIN_END;
	exit( res );
}
