/*****************************************************************************
 * $LastChangedDate: 2008-05-13 23:19:41 -0400 (Tue, 13 May 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 *//*
 * LEGAL:	COPYRIGHT (C) 2005 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef TEST_HH
#define TEST_HH

// Don't tell Bjarne ;-)
#define private public

// To test test internal functions.
#define INTERN
#define INTERNAL

//#undef DEBUG
//#define DEBUG  1


// Put both of these in main() of every test.
// base::gGlobals.mTesting affects how assertions and exceptions are raised.
#define TEST_MAIN_BEGIN {{ base::gGlobals.mTesting = true; }}
#define TEST_MAIN_END

//..............................................................................
// SharedPtr methods contain the macro TEST_SHARED_PTR_YIELD_CPU()
// to help better test lock contention.
extern void TestYield( void );
#define TEST_SHARED_PTR_YIELD_CPU()  TestYield()
//..............................................................................

#include "base/module.hh"
using namespace base;

#if COMPILE_THREADS
void TestYield( void )
{
	gThread.Yield();
}
#endif // COMPILE_THREADS

/*****************************************************************************
 * __LINE__ is apparently an int, not a string.
 *
 * Example:
 * if ( cond ) return Fail();
 *****************************************************************************/
bool
Fail_( int line )
{
	CDEBUG << "FAILED at line=" << line << endl;
	_exit( 1 );  // really exit
	return false;
}
#define Fail() Fail_( __LINE__ )

// Lock stream to avoid intermixing output.
// Printing cause delays that makes the test unrealistic.
#ifndef VERBOSE
#define VERBOSE  1
#endif

FastLock sLockStream;
#define LOCK_STREAM( STREAM_CODE ) {{ sLockStream.Lock(); STREAM_CODE; sLockStream.Unlock(); }}

#if VERBOSE
#define VERBOSE_CODE( STREAM_CODE ) LOCK_STREAM( STREAM_CODE )
#else
#define VERBOSE_CODE( STREAM_CODE )
#endif

#endif // TEST_HH
